==> SAGElyzer_1.3.0.tar.gz: SAGElyzer/R/SAGELyzer.R <==
# These functions processes the SAGE library data and find genes that
# are similar to a given SAGE tag based on specified criteria.
#
# Copyright 2002, J. Zhang. All rights reserved.
#

SAGELyzer <- function(dbArgs, targetSAGE, libs = "*", normalize = "min",
                      tagColName = "tag", k = 500, dist = "euclidean",
                      trans = "sqrt"){
    # genefinder of genefilter is needed
    require(genefilter) || stop("Library genefilter unavailable!")
    # Argument trans must be a function
    if( is.character(trans) )
        trans <- get(trans, mode="function")
    if( !is.function(trans) )
        stop("trans must be a function")
    # Distances of genes to the target gene will be keeped and
    # returned. k defines how many of the most similar genes will be kept
    distance <- NULL
    # Needs a target gene to work with
    if(missing(targetSAGE) || is.null(targetSAGE)){
         tkmessageBox(title = "Query error", message = paste("Target tage",
                     "is either null or missing!"),
                     icon = "warning", type = "ok")
         return(NULL)
    }else{
        # Get K nearest neighbours
        neighbors <- getKNN(dbArgs, targetSAGE, libs, tagColName,
                            normalize, k, dist, trans)
        return(neighbors)
    }
}

# Get counts for the target row across selected libraries
getTargetRow <- function (dbArgs, conn, libs, tagColName, targetSAGE,
                          what = "counts"){
    if(length(libs) > 1){
        columns <- paste(c(tagColName, libs), sep = "",collapse = ",")
    }else{
        columns <- libs
    }

    targetSQL <- paste("select ", paste(columns, sep = "",
                                        collapse = ","), " from ",
                       dbArgs[[what]], " where ", tagColName,
                       " = '", targetSAGE, "'", sep = "")
    targetLine <- executeQuery(targetSQL, conn)

    if(is.null(targetLine) || is.na(targetLine) ||
       nrow(targetLine) == 0){
        tkmessageBox(title = "Query error", message = paste("Target tage",
                     targetSAGE, "does not exist in the DB"),
                     icon = "warning", type = "ok")
        return(NULL)
    }else{
        return(targetLine)
    }
}

# This function gets the normalization factor
getNormFactor <- function(normalize = c("min", "max", "none"), libs){
    normalize <- match.arg(normalize)
    switch(normalize,
           none = return(NULL),
           min = NF <- queryInfoDB(infoCol = "minfactor"),
           max = NF <- queryInfoDB(infoCol = "maxfactor"),
           stop("Invalid argument for normalize. Must be min, max, or none"))
    if(libs[1] == "*"){
        return(NF)
    }else{
        return(NF[libs])
    }
}

# This function queries sageinfo table to get file name or
# normalization factor for each library coded as lib1, lib2, ...
queryInfoDB <- function(libCol = "libname",
                        infoCol = c("filename", "minfactor", "maxfactor")){
    infoCol <- match.arg(infoCol)
    dbArgs <- getSLyzerArgs()$dbArgs
    switch(infoCol,
           filename = ,
           minfactor = ,
           maxfactor = NFSQL <- paste("select ", libCol, ",", infoCol,
                                      " from ", dbArgs[["info"]]),
           stop("Invalid argument for factName"))
    conn <- makeConnection(dbArgs)
    info <- executeQuery(NFSQL, conn)
    temp <- info[,2]
    names(temp) <- info[,1]
#    closeConn(conn)
    return(temp)
}

## This function is not in use any more but is kept anyway in case.
#calNormFactor <- function(normalize, dbArgs, conn, libs, tagColName){
#    if(!is.na(match("*", libs))){
#        libs <- setdiff(getColNames(dbArgs, conn), tagColName)
#    }

#    libNNum <- NULL
    # Has to do it for each library as the table can be very large and
    # may not be feasible to select all libraries once and figure
    #out the number of entries for each
#    for(i in libs){
#        factSQL <- paste("select count(", i, ") from ", dbArgs[["tableName"]],
#                         " where ", i, " > 0", sep = "")
#        got <- executeQuery(factSQL, conn)
#        libNNum <- c(libNNum, i, unlist(got, use.name = FALSE))
#    }
#    libNNum <- matrix(libNNum, ncol = 2, byrow = TRUE)
#    if(normalize == "min"){
#        temp <- min(as.numeric(libNNum[,2]))
#    }else{
#        temp <- max(as.numeric(libNNum[,2]))
#    }
#    factor <- as.numeric(temp/as.numeric(libNNum[,2]))
#    names(factor) <- libNNum[,1]
#    return(factor)
#}

# Get the total number of rows from an existing database table.
getTotalRNum <- function(dbArgs, conn, tagColName, what = "counts"){
    rows <- executeQuery(paste("select count(", tagColName, ") from ",
                               dbArgs[[what]], sep = ""), conn)
    if(!is.null(rows))
        return(rows)
    else
        return(null)
}

# Get k nearest neighbours for the target SAGE
getKNN <- function(dbArgs, targetSAGE, libs, tagColName, normalize,
                   k, dist, trans, max = 10000){
    conn <- makeConnection(dbArgs)
    options(show.error.messages = FALSE)
    rowNum <- try(getTotalRNum(dbArgs, conn, tagColName))
    options(show.error.messages = TRUE)
    if(inherits(rowNum, "try-error")){
        clossConn(conn)
        print("Failed to get total row number from DB")
    }else{
        if(rowNum < max){
            options(show.error.messages = FALSE)
            neighbors <- try(noChunkKNN(dbArgs, conn, targetSAGE, libs,
                                    tagColName, normalize, k, dist, trans))
            options(show.error.messages = TRUE)
        }else{
            options(show.error.messages = FALSE)
            neighbors <- try(chunkKNN(dbArgs, conn, targetSAGE, libs,
                                      tagColName, normalize, k, dist,
                                      trans, rowNum, max = max))
            options(show.error.messages = TRUE)
        }
    }
#    closeConn(conn)
    if(inherits(neighbors, "try-error")){
        stop("Failed to find the k neighbors")
    }else{
        return(neighbors)
    }
}

# Do this if total number of rows is less than MAX
noChunkKNN <- function(dbArgs, conn, targetSAGE, libs, tagColName,
                       normalize, k, dist, trans){
    distance <- NULL
    # Get the normorlization factor
#    print("Calculating normalization factors")
    NF <- getNormFactor(normalize, libs)
    # Gets the values across libraries for the target SAGE
#    print("Getting libraries for the target SAGE tag")
    targetRow <- getTargetRow(dbArgs, conn, libs, tagColName,
                              targetSAGE)
    if(!is.null(targetRow)){
#        print("Calculating distances")
        knnSQL <- getSAGESQL(dbArgs, conn, targetSAGE, libs, tagColName,
                             what = "counts")
        dataRead <- executeQuery(knnSQL, conn)

        tempDist <- findNeighborTags(targetRow, dataRead, k, NF, dist, trans)
        # Sorts the distance from closest to farthest
        distance <- sort(c(distance, tempDist))
        # Trim distance values to the defined length k
        if(length(distance) > k){
            distance <- distance[1:k]
        }
        return(distance)
    }
}

# Do this if the total number of rows is greater than
chunkKNN <- function(dbArgs, conn, targetSAGE, libs, tagColName, normalize,
                     k, dist, trans, rowNum, max = 50000){

    # Distances of genes to the target gene will be keeped and
    # returned. k defines how many of the most similar genes will be kept
    distance <- NULL
    # Get the normorlization factor
#    print("Calculating normalization factors")
    NF <- getNormFactor(normalize, libs)
    # Gets the values across libraries for the target SAGE
#    print("Getting libraries for the target SAGE")
    targetRow <- getTargetRow(dbArgs, conn, libs, tagColName, targetSAGE)
    if(!is.null(targetRow)){
#        print("Calculating distances")
        executeQuery("begin", conn, TRUE)
        cursor <- "sageRows"
        knnSQL <- getSAGESQL(dbArgs, conn, targetSAGE, libs, tagColName,
                         chunk = TRUE, cursor = cursor, what = "counts")
        executeQuery(knnSQL, conn, TRUE)
        for(i in 1:ceiling(rowNum/max)[[1]]){
            dataRead <- executeQuery(paste("fetch forward", max, "in",
                                           cursor), conn, FALSE)
            # Chunking is normally less than ceiling(rowNum/MAX)[[1]]
            # due to the removal of 0 rows. Check here
            if(nrow(dataRead) == 0)
                break

            tempDist <- findNeighborTags(targetRow, dataRead, k, NF,
                                         dist, trans)
            distance <- sort(c(distance, tempDist))
            # Remove target tag
            distance <- distance[names(distance) != targetSAGE]
            # Trim distance values to the defined length k
            if(length(distance) > k){
                distance <- distance[1:k]
            }
        }
        executeQuery(paste("close", cursor), conn, TRUE)
        executeQuery("end", conn, TRUE)

        return(distance)
    }
}

# Get the distance of entries in the data from the target row
findNeighborTags <- function(targetRow, data, k, NF, dist, trans){

    # Append the target row to the end
    data <- rbind(data, targetRow)
#    data[(nrow(data) + 1),] <- unlist(targetRow, use.name = FALSE)
    # The first column is tag that is not needed for distance calculation
    temp <- as.matrix(data[, 2:ncol(data)])
    # Puts 0 if any of the cells in the matrix is NA
    temp[is.na(temp)] <- 0
    # Normalize the data with the normalization factor
    if(!is.null(NF)){
        # Data returned from db query are converted to character.
        # Convert to numeric now
        temp <- matrix(as.numeric(temp), ncol = ncol(temp), byrow = FALSE)
        temp <- sweep(temp, 2, NF, "/")
    }
    # Gets the distance and other variables using genefinder
    found <- genefinder(trans(temp), nrow(temp),
                        ifelse(nrow(temp) > k, k, nrow(temp)),
                        method = dist)
    # Extracts the distance
    tempDist <- as.numeric(found[[1]]$dists)
    # Gets the name for each distance
    names(tempDist) <- data[found[[1]]$indices,][[1]]

    return(tempDist)
}

# Get the SQL statement for the query to get data for required
# libraries excluding the target row and row with all 0s if ignorZeros
# = TRUE
getSAGESQL <- function(dbArgs, conn, targetSAGE, libs, tagColName,
                       chunk = FALSE, cursor = "sageRows",
                       ignorZeros = TRUE,
                       what = c("map", "counts", "info")){
    what <- match.arg(what)

    if(ignorZeros){
        if(!is.na(match("*", libs))){
            libs <- setdiff(getColNames(dbArgs, conn), tagColName)
        }
#        toAdd <- paste(" and (", paste(libs, sep = "", collapse = "+"),
#                           ") > 0", sep = "")
        # This will work for both numeric or character data type
        toAdd <- paste(" and (", paste(libs, "<> '0'",
                                       collapse = " or "), ")", sep = "")
    }

    if(length(libs) > 1){
        columns <- paste(c(tagColName, libs), sep = "",collapse = ",")
    }else{
        columns <- libs
    }

    switch(what,
           map = ,
           counts = ,
           info = sageSQL <- paste("select ", columns, " from ",
                                   dbArgs[[what]], " where ",
                                   tagColName, " <> '", targetSAGE, "'",
                                   toAdd, sep = ""),
           stop("Invalid argument for what"))
    if(chunk){
        sageSQL <- paste("declare", cursor, "cursor for", sageSQL)
    }
    return(sageSQL)
}

# Gets all the column names of a table
getColNames <- function(dbArgs, conn, what = "counts"){
    colSQL <- paste("select * from", dbArgs[[what]], "limit 1")
    colNames <- executeQuery(colSQL, conn, FALSE)
    return(names(colNames))
}

==> SAGElyzer_1.10.0.tar.gz: SAGElyzer/R/SAGELyzer.R <==
# These functions processes the SAGE library data and find genes that
# are similar to a given SAGE tag based on specified criteria.
#
# Copyright 2002, J. Zhang. All rights reserved.
#

SAGELyzer <- function(dbArgs, targetSAGE, libs = "*", normalize = "min",
                      tagColName = "tag", k = 500, dist = "euclidean",
                      trans = "sqrt"){
    # genefinder of genefilter is needed
    require("genefilter", character.only = TRUE) || 
        stop("Library genefilter unavailable!")
    # Argument trans must be a function
    if( is.character(trans) )
        trans <- get(trans, mode="function")
    if( !is.function(trans) )
        stop("trans must be a function")
    # Distances of genes to the target gene will be keeped and
    # returned. k defines how many of the most similar genes will be kept
    distance <- NULL
    # Needs a target gene to work with
    if(missing(targetSAGE) || is.null(targetSAGE)){
         tkmessageBox(title = "Query error", message = paste("Target tage",
                     "is either null or missing!"),
                     icon = "warning", type = "ok")
         return(NULL)
    }else{
        # Get K nearest neighbours
        neighbors <- getKNN(dbArgs, targetSAGE, libs, tagColName,
                            normalize, k, dist, trans)
        return(neighbors)
    }
}

# Get counts for the target row across selected libraries
getTargetRow <- function (dbArgs, conn, libs, tagColName, targetSAGE,
                          what = "counts"){
    if(length(libs) > 1){
        columns <- paste(c(tagColName, libs), sep = "",collapse = ",")
    }else{
        columns <- libs
    }

    targetSQL <- paste("select ", paste(columns, sep = "",
                                        collapse = ","), " from ",
                       dbArgs[[what]], " where ", tagColName,
                       " = '", targetSAGE, "'", sep = "")
    targetLine <- executeQuery(targetSQL, conn)

    if(is.null(targetLine) || is.na(targetLine) ||
       nrow(targetLine) == 0){
        tkmessageBox(title = "Query error", message = paste("Target tage",
                     targetSAGE, "does not exist in the DB"),
                     icon = "warning", type = "ok")
        return(NULL)
    }else{
        return(targetLine)
    }
}

# This function gets the normalization factor
getNormFactor <- function(normalize = c("min", "max", "none"), libs){
    normalize <- match.arg(normalize)
    switch(normalize,
           none = return(NULL),
           min = NF <- queryInfoDB(infoCol = "minfactor"),
           max = NF <- queryInfoDB(infoCol = "maxfactor"),
           stop("Invalid argument for normalize. Must be min, max, or none"))
    if(libs[1] == "*"){
        return(NF)
    }else{
        return(NF[libs])
    }
}

# This function queries sageinfo table to get file name or
# normalization factor for each library coded as lib1, lib2, ...
queryInfoDB <- function(libCol = "libname",
                        infoCol = c("filename", "minfactor", "maxfactor")){
    infoCol <- match.arg(infoCol)
    dbArgs <- getSLyzerArgs()$dbArgs
    switch(infoCol,
           filename = ,
           minfactor = ,
           maxfactor = NFSQL <- paste("select ", libCol, ",", infoCol,
                                      " from ", dbArgs[["info"]]),
           stop("Invalid argument for factName"))
    conn <- makeConnection(dbArgs)
    info <- executeQuery(NFSQL, conn)
    temp <- info[,2]
    names(temp) <- info[,1]
#    closeConn(conn)
    return(temp)
}

## This function is not in use any more but is kept anyway in case.
#calNormFactor <- function(normalize, dbArgs, conn, libs, tagColName){
#    if(!is.na(match("*", libs))){
#        libs <- setdiff(getColNames(dbArgs, conn), tagColName)
#    }

#    libNNum <- NULL
    # Has to do it for each library as the table can be very large and
    # may not be feasible to select all libraries once and figure
    #out the number of entries for each
#    for(i in libs){
#        factSQL <- paste("select count(", i, ") from ", dbArgs[["tableName"]],
#                         " where ", i, " > 0", sep = "")
#        got <- executeQuery(factSQL, conn)
#        libNNum <- c(libNNum, i, unlist(got, use.name = FALSE))
#    }
#    libNNum <- matrix(libNNum, ncol = 2, byrow = TRUE)
#    if(normalize == "min"){
#        temp <- min(as.numeric(libNNum[,2]))
#    }else{
#        temp <- max(as.numeric(libNNum[,2]))
#    }
#    factor <- as.numeric(temp/as.numeric(libNNum[,2]))
#    names(factor) <- libNNum[,1]
#    return(factor)
#}

# Get the total number of rows from an existing database table.
getTotalRNum <- function(dbArgs, conn, tagColName, what = "counts"){
    rows <- executeQuery(paste("select count(", tagColName, ") from ",
                               dbArgs[[what]], sep = ""), conn)
    if(!is.null(rows))
        return(rows)
    else
        return(null)
}

# Get k nearest neighbours for the target SAGE
getKNN <- function(dbArgs, targetSAGE, libs, tagColName, normalize,
                   k, dist, trans, max = 10000){
    conn <- makeConnection(dbArgs)
    options(show.error.messages = FALSE)
    rowNum <- try(getTotalRNum(dbArgs, conn, tagColName))
    options(show.error.messages = TRUE)
    if(inherits(rowNum, "try-error")){
        closeConn(conn)
        print("Failed to get total row number from DB")
    }else{
        if(rowNum < max){
            options(show.error.messages = FALSE)
            neighbors <- try(noChunkKNN(dbArgs, conn, targetSAGE, libs,
                                    tagColName, normalize, k, dist, trans))
            options(show.error.messages = TRUE)
        }else{
            options(show.error.messages = FALSE)
            neighbors <- try(chunkKNN(dbArgs, conn, targetSAGE, libs,
                                      tagColName, normalize, k, dist,
                                      trans, rowNum, max = max))
            options(show.error.messages = TRUE)
        }
    }
#    closeConn(conn)
    if(inherits(neighbors, "try-error")){
        stop("Failed to find the k neighbors")
    }else{
        return(neighbors)
    }
}

# Do this if total number of rows is less than MAX
noChunkKNN <- function(dbArgs, conn, targetSAGE, libs, tagColName,
                       normalize, k, dist, trans){
    distance <- NULL
    # Get the normorlization factor
#    print("Calculating normalization factors")
    NF <- getNormFactor(normalize, libs)
    # Gets the values across libraries for the target SAGE
#    print("Getting libraries for the target SAGE tag")
    targetRow <- getTargetRow(dbArgs, conn, libs, tagColName,
                              targetSAGE)
    if(!is.null(targetRow)){
#        print("Calculating distances")
        knnSQL <- getSAGESQL(dbArgs, conn, targetSAGE, libs, tagColName,
                             what = "counts")
        dataRead <- executeQuery(knnSQL, conn)

        tempDist <- findNeighborTags(targetRow, dataRead, k, NF, dist, trans)
        # Sorts the distance from closest to farthest
        distance <- sort(c(distance, tempDist))
        # Trim distance values to the defined length k
        if(length(distance) > k){
            distance <- distance[1:k]
        }
        return(distance)
    }
}

# Do this if the total number of rows is greater than
chunkKNN <- function(dbArgs, conn, targetSAGE, libs, tagColName, normalize,
                     k, dist, trans, rowNum, max = 50000){

    # Distances of genes to the target gene will be keeped and
    # returned. k defines how many of the most similar genes will be kept
    distance <- NULL
    # Get the normorlization factor
#    print("Calculating normalization factors")
    NF <- getNormFactor(normalize, libs)
    # Gets the values across libraries for the target SAGE
#    print("Getting libraries for the target SAGE")
    targetRow <- getTargetRow(dbArgs, conn, libs, tagColName, targetSAGE)
    if(!is.null(targetRow)){
#        print("Calculating distances")
        executeQuery("begin", conn, TRUE)
        cursor <- "sageRows"
        knnSQL <- getSAGESQL(dbArgs, conn, targetSAGE, libs, tagColName,
                         chunk = TRUE, cursor = cursor, what = "counts")
        executeQuery(knnSQL, conn, TRUE)
        for(i in 1:ceiling(rowNum/max)[[1]]){
            dataRead <- executeQuery(paste("fetch forward", max, "in",
                                           cursor), conn, FALSE)
            # Chunking is normally less than ceiling(rowNum/MAX)[[1]]
            # due to the removal of 0 rows. Check here
            if(nrow(dataRead) == 0)
                break

            tempDist <- findNeighborTags(targetRow, dataRead, k, NF,
                                         dist, trans)
            distance <- sort(c(distance, tempDist))
            # Remove target tag
            distance <- distance[names(distance) != targetSAGE]
            # Trim distance values to the defined length k
            if(length(distance) > k){
                distance <- distance[1:k]
            }
        }
        executeQuery(paste("close", cursor), conn, TRUE)
        executeQuery("end", conn, TRUE)

        return(distance)
    }
}

# Get the distance of entries in the data from the target row
findNeighborTags <- function(targetRow, data, k, NF, dist, trans){

    # Append the target row to the end
    data <- rbind(data, targetRow)
#    data[(nrow(data) + 1),] <- unlist(targetRow, use.name = FALSE)
    # The first column is tag that is not needed for distance calculation
    temp <- as.matrix(data[, 2:ncol(data)])
    # Puts 0 if any of the cells in the matrix is NA
    temp[is.na(temp)] <- 0
    # Normalize the data with the normalization factor
    if(!is.null(NF)){
        # Data returned from db query are converted to character.
        # Convert to numeric now
        temp <- matrix(as.numeric(temp), ncol = ncol(temp), byrow = FALSE)
        temp <- sweep(temp, 2, NF, "/")
    }
    # Gets the distance and other variables using genefinder
    found <- genefinder(trans(temp), nrow(temp),
                        ifelse(nrow(temp) > k, k, nrow(temp)),
                        method = dist)
    # Extracts the distance
    tempDist <- as.numeric(found[[1]]$dists)
    # Gets the name for each distance
    names(tempDist) <- data[found[[1]]$indices,][[1]]

    return(tempDist)
}

# Get the SQL statement for the query to get data for required
# libraries excluding the target row and row with all 0s if ignorZeros
# = TRUE
getSAGESQL <- function(dbArgs, conn, targetSAGE, libs, tagColName,
                       chunk = FALSE, cursor = "sageRows",
                       ignorZeros = TRUE,
                       what = c("map", "counts", "info")){
    what <- match.arg(what)

    if(ignorZeros){
        if(!is.na(match("*", libs))){
            libs <- setdiff(getColNames(dbArgs, conn), tagColName)
        }
#        toAdd <- paste(" and (", paste(libs, sep = "", collapse = "+"),
#                           ") > 0", sep = "")
        # This will work for both numeric or character data type
        toAdd <- paste(" and (", paste(libs, "<> '0'",
                                       collapse = " or "), ")", sep = "")
    }

    if(length(libs) > 1){
        columns <- paste(c(tagColName, libs), sep = "",collapse = ",")
    }else{
        columns <- libs
    }

    switch(what,
           map = ,
           counts = ,
           info = sageSQL <- paste("select ", columns, " from ",
                                   dbArgs[[what]], " where ",
                                   tagColName, " <> '", targetSAGE, "'",
                                   toAdd, sep = ""),
           stop("Invalid argument for what"))
    if(chunk){
        sageSQL <- paste("declare", cursor, "cursor for", sageSQL)
    }
    return(sageSQL)
}

# Gets all the column names of a table
getColNames <- function(dbArgs, conn, what = "counts"){
    colSQL <- paste("select * from", dbArgs[[what]], "limit 1")
    colNames <- executeQuery(colSQL, conn, FALSE)
    return(names(colNames))
}
